<?php declare(strict_types=1);

namespace TurnstileReMaster\Migration; // Stelle sicher, dass der Namespace korrekt ist

use Doctrine\DBAL\Connection;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Migration\MigrationStep;
use Shopware\Core\Framework\Uuid\Uuid; // Importieren für `created_at` falls nicht vorhanden
use Shopware\Core\System\SystemConfig\SystemConfigService; // Kann nützlich sein, aber hier reicht Connection

class Migration1664374217addTurnStileCaptcha extends MigrationStep
{
    // Der System-Config Key für die Captchas
    private const CONFIG_KEY = 'core.basicInformation.activeCaptchasV2';
    // Der eindeutige Key für unser Captcha innerhalb der Konfiguration
    private const CLOUDFLARE_CAPTCHA_KEY = 'cloudFlareTurnstile';

    public function getCreationTimestamp(): int
    {
        return 1664374217; // Behalte den ursprünglichen Timestamp bei
    }

    /**
     * Gibt die Standardkonfiguration für Cloudflare Turnstile zurück.
     *
     * @return array<string, mixed>
     */
    private function getCloudflareDefaultConfig(): array
    {
        return [
            'name' => self::CLOUDFLARE_CAPTCHA_KEY, // Oder 'Cloudflare Turnstile' für den Anzeigenamen
            'isActive' => false, // Standardmäßig deaktiviert
            'config' => [
                'siteKey' => '',
                'secretKey' => '',
            ],
        ];
    }

    public function update(Connection $connection): void
    {
        // Hole die aktuelle globale Konfiguration (sales_channel_id IS NULL)
        $currentConfig = $connection->fetchAssociative(
            'SELECT id, configuration_value FROM system_config WHERE configuration_key = :key AND sales_channel_id IS NULL LIMIT 1',
            ['key' => self::CONFIG_KEY]
        );

        // Fall 1: Der Basiseintrag für Captchas existiert noch gar nicht (sehr unwahrscheinlich im Core, aber sicher ist sicher)
        if ($currentConfig === false) {
            // Erstelle einen neuen Eintrag mit allen Captchas (dies sollte selten passieren)
            // Holen der Standard-Captchas von Shopware wäre hier besser, aber für den Moment erstellen wir nur mit unserem
            $allCaptchas = $this->getDefaultShopwareCaptchas(); // Hole Standard-Captchas (siehe Helper unten)
            $allCaptchas[self::CLOUDFLARE_CAPTCHA_KEY] = $this->getCloudflareDefaultConfig();

            $connection->insert('system_config', [
                'id' => Uuid::randomBytes(),
                'configuration_key' => self::CONFIG_KEY,
                'configuration_value' => json_encode(['_value' => $allCaptchas]),
                'sales_channel_id' => null,
                'created_at' => (new \DateTime())->format(Defaults::STORAGE_DATE_TIME_FORMAT),
            ]);
            return; // Fertig
        }

        $configId = $currentConfig['id'];
        $currentValueJson = $currentConfig['configuration_value'];

        // Fall 2: Eintrag existiert, aber Wert ist leer oder ungültig
        if (empty($currentValueJson)) {
            $configValue = ['_value' => []]; // Start mit leerem Array
        } else {
            $configValue = json_decode($currentValueJson, true);
            // Prüfen, ob JSON-Dekodierung fehlschlug oder die Struktur nicht passt
            if (json_last_error() !== JSON_ERROR_NONE || !isset($configValue['_value']) || !is_array($configValue['_value'])) {
                // Logge einen Fehler oder überschreibe mit sinnvollem Standard? Fürs Erste überspringen wir.
                // Alternativ: $configValue = ['_value' => []]; // Reset bei ungültiger Struktur
                // Oder wir loggen es und tun nichts: error_log('Invalid JSON structure for ' . self::CONFIG_KEY);
                return; // Änderung nicht sicher möglich
            }
        }

        // Fall 3: Eintrag existiert und ist gültig - Prüfen, ob unser Captcha schon da ist
        if (isset($configValue['_value'][self::CLOUDFLARE_CAPTCHA_KEY])) {
            // Unser Captcha ist bereits vorhanden, nichts zu tun.
            return;
        }

        // Fall 4: Unser Captcha hinzufügen
        $configValue['_value'][self::CLOUDFLARE_CAPTCHA_KEY] = $this->getCloudflareDefaultConfig();

        // Die aktualisierte Konfiguration zurück in die Datenbank schreiben
        $connection->update(
            'system_config',
            [
                // JSON mit korrektem Wrapping (_value) speichern
                'configuration_value' => json_encode($configValue),
                // Wichtig: updated_at setzen, nicht created_at!
                'updated_at' => (new \DateTime())->format(Defaults::STORAGE_DATE_TIME_FORMAT),
            ],
            [
                // Sicherstellen, dass wir den korrekten Datensatz aktualisieren
                'id' => $configId,
            ]
        );
    }

    /**
     * Helper, um die Standard-Captcha-Liste zu bekommen (vereinfacht).
     * In einer echten Migration könnte man versuchen, diese dynamischer zu laden,
     * aber für den Zweck der Migration ist eine feste Liste oft ausreichend.
     * WICHTIG: Diese Liste muss ggf. an die Ziel-Shopware-Version angepasst werden!
     * Prüfe die `shopware/core/System/SystemConfig/Service/ConfigurationService.php`
     * oder ähnliche Core-Dateien der Zielversion für die korrekten Standard-Captchas.
     *
     * @return array<string, mixed>
     */
    private function getDefaultShopwareCaptchas(): array
    {
        // Diese Liste basiert auf deiner alten Migration und muss ggf. für SW 6.6 geprüft/angepasst werden!
        // Shopware 6.6 enthält möglicherweise andere/mehr Standard-Captchas.
        return [
            'honeypot' => [
                'name' => 'Honeypot',
                'isActive' => true, // Honeypot ist oft standardmäßig aktiv
            ],
            'basicCaptcha' => [
                'name' => 'basicCaptcha',
                'isActive' => false,
            ],
            'googleReCaptchaV2' => [
                'name' => 'googleReCaptchaV2',
                'isActive' => false,
                'config' => [
                    'siteKey' => '',
                    'secretKey' => '',
                    'invisible' => false,
                ],
            ],
            'googleReCaptchaV3' => [
                'name' => 'googleReCaptchaV3',
                'isActive' => false,
                'config' => [
                    'siteKey' => '',
                    'secretKey' => '',
                    'thresholdScore' => 0.5,
                ],
            ],
            // Füge hier ggf. weitere Standard-Captchas von SW 6.6 hinzu
        ];
    }


    public function updateDestructive(Connection $connection): void
    {
        // Implementiere, falls nötig (z.B. Entfernen alter Konfigurationen)
        // Normalerweise hier nichts zu tun für das Hinzufügen eines Captchas.
    }
}