<?php declare(strict_types=1);

namespace TurnstileReMaster\Storefront\Framework\Captcha; // Stelle sicher, dass der Namespace korrekt ist

use GuzzleHttp\ClientInterface;
use Psr\Http\Client\ClientExceptionInterface;
use Psr\Log\LoggerInterface; // Logger hinzufügen
use Shopware\Storefront\Framework\Captcha\AbstractCaptcha;
use Symfony\Component\HttpFoundation\Request;
use Throwable; // Für generischere Exception-Behandlung

/**
 * @internal // Beibehalten, wenn es eine interne Klasse des Plugins ist
 */
final class CloudFlareTurnstile extends AbstractCaptcha // 'final', wenn keine Ableitung geplant ist
{
    public const CAPTCHA_NAME = 'cloudFlareTurnstile';
    // Injected by CF in form
    public const CAPTCHA_REQUEST_PARAMETER = 'cf-turnstile-response';
    private const CLOUDFLARE_CAPTCHA_VERIFY_ENDPOINT = 'https://challenges.cloudflare.com/turnstile/v0/siteverify';

    private ClientInterface $client;
    private LoggerInterface $logger; // Logger-Instanz

    /**
     * @internal
     */
    public function __construct(
        ClientInterface $client,
        LoggerInterface $logger // Logger injizieren
    ) {
        $this->client = $client;
        $this->logger = $logger;
    }

    /**
     * {@inheritdoc}
     * Validiert das Cloudflare Turnstile Captcha.
     * Erwartet die Konfiguration im $captchaConfig Array.
     */
    public function isValid(Request $request, array $captchaConfig): bool // Klare Signatur für SW >= 6.5
    {
        $turnstileResponse = $request->request->get(self::CAPTCHA_REQUEST_PARAMETER); // Sicherer Zugriff auf POST-Parameter

        if (!\is_string($turnstileResponse) || $turnstileResponse === '') {
            $this->logger->debug('Cloudflare Turnstile token missing in request.', ['request_params' => $request->request->all()]);
            return false;
        }

        // Hole Secret Key aus der Konfiguration
        $secretKey = $captchaConfig['config']['secretKey'] ?? null;

        if (!\is_string($secretKey) || $secretKey === '') {
            $this->logger->error('Cloudflare Turnstile secret key is missing or not configured in Shopware settings.');
            // Gib false zurück, da ohne Secret Key keine Validierung möglich ist.
            // Alternativ könnte man hier eine Exception werfen, aber false ist im Sinne von isValid() oft besser.
            return false;
        }

        try {
            $response = $this->client->request('POST', self::CLOUDFLARE_CAPTCHA_VERIFY_ENDPOINT, [
                'form_params' => [
                    'secret' => $secretKey,
                    'response' => $turnstileResponse,
                    'remoteip' => $request->getClientIp(), // Client IP mitsenden ist optional aber empfohlen
                ],
                'timeout' => 5, // Timeout hinzufügen, um lange Wartezeiten zu vermeiden
            ]);

            $responseBody = $response->getBody()->getContents();
            $responseData = json_decode($responseBody, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                $this->logger->warning('Failed to decode Cloudflare Turnstile verify response.', ['response_body' => $responseBody]);
                return false;
            }

            if (!isset($responseData['success'])) {
                $this->logger->warning('Cloudflare Turnstile verify response missing "success" key.', ['response_data' => $responseData]);
                return false;
            }

            if ($responseData['success'] === true) {
                return true; // Captcha ist gültig
            }

            // Logge Fehlercodes von Cloudflare, falls vorhanden
            $errorCodes = $responseData['error-codes'] ?? [];
            $this->logger->info('Cloudflare Turnstile validation failed.', [
                'success' => false,
                'error-codes' => $errorCodes,
                'clientIp' => $request->getClientIp() // Logge relevante Infos
            ]);
            return false; // Captcha ist ungültig

        } catch (ClientExceptionInterface $e) {
            // Fehler bei der HTTP-Anfrage an Cloudflare (Netzwerkproblem, Timeout etc.)
            $this->logger->error('HTTP error during Cloudflare Turnstile verification.', [
                'exception' => $e->getMessage(),
                'trace' => $e->getTraceAsString() // Optional für Debugging
            ]);
            return false; // Bei Fehlern als ungültig betrachten
        } catch (Throwable $e) {
            // Fange andere mögliche Fehler ab (z.B. Guzzle-Fehler, JSON-Fehler etc.)
            $this->logger->error('Unexpected error during Cloudflare Turnstile verification.', [
                'exception' => $e->getMessage(),
                'trace' => $e->getTraceAsString()
            ]);
            return false;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getName(): string
    {
        return self::CAPTCHA_NAME;
    }
}